import {useEffect, useRef, useState} from "react";
import "./styles.sass";

export const MusicSlider = ({ isReadOnly = false, onFrequencyChange, frequency: externalFrequency }) => {
    const [frequency, setFrequency] = useState(externalFrequency || 440);
    const [isPlaying, setIsPlaying] = useState(false);
    const [dragging, setDragging] = useState(false);
    const audioContextRef = useRef(null);
    const oscillatorRef = useRef(null);
    const gainNodeRef = useRef(null);
    const sliderRef = useRef(null);

    useEffect(() => {
        if (externalFrequency !== undefined && !dragging) {
            setFrequency(externalFrequency);

            if (!isPlaying && !isReadOnly) {
                startAudio();
            }
        }
    }, [externalFrequency, dragging, isPlaying, isReadOnly]);

    const handleMouseDown = (e) => {
        if (isReadOnly) return;
        
        setDragging(true);
        startAudio();
        handleFrequencyChange(e);
    };

    const handleMouseUp = () => {
        setDragging(false);
        if (!isReadOnly) {
            stopAudio();
        }
    };

    const handleFrequencyChange = (e) => {
        if (isReadOnly) return;
        
        const rect = sliderRef.current.getBoundingClientRect();
        const x = e.clientX - rect.left;
        const clampedX = Math.max(0, Math.min(x, rect.width));
        const newFrequency = 20 + (clampedX / rect.width) * 1980;
        setFrequency(newFrequency);

        if (onFrequencyChange) {
            onFrequencyChange(newFrequency);
        }
    };

    useEffect(() => {
        const handleMouseMove = (e) => dragging && handleFrequencyChange(e);
        const handleMouseUpGlobal = () => dragging && handleMouseUp();

        document.addEventListener('mousemove', handleMouseMove);
        document.addEventListener('mouseup', handleMouseUpGlobal);

        return () => {
            document.removeEventListener('mousemove', handleMouseMove);
            document.removeEventListener('mouseup', handleMouseUpGlobal);
        };
    }, [dragging]);

    useEffect(() => {
        if (isPlaying) {
            if (oscillatorRef.current) {
                oscillatorRef.current.frequency.setValueAtTime(frequency, audioContextRef.current.currentTime);
            }
        }
    }, [frequency, isPlaying]);

    const startAudio = () => {
        if (!audioContextRef.current) {
            audioContextRef.current = new (window.AudioContext || window.webkitAudioContext)();
        }
        if (!oscillatorRef.current) {
            oscillatorRef.current = audioContextRef.current.createOscillator();
            oscillatorRef.current.type = 'sine';
            oscillatorRef.current.frequency.setValueAtTime(frequency, audioContextRef.current.currentTime);

            gainNodeRef.current = audioContextRef.current.createGain();
            gainNodeRef.current.gain.setValueAtTime(0.5, audioContextRef.current.currentTime);

            oscillatorRef.current.connect(gainNodeRef.current);
            gainNodeRef.current.connect(audioContextRef.current.destination);
            oscillatorRef.current.start();
        } else {
            oscillatorRef.current.frequency.setValueAtTime(frequency, audioContextRef.current.currentTime);
        }
        setIsPlaying(true);
    };

    const stopAudio = () => {
        if (oscillatorRef.current) {
            oscillatorRef.current.stop();
            oscillatorRef.current.disconnect();
            oscillatorRef.current = null;
        }
        setIsPlaying(false);
    };

    useEffect(() => {
        return () => {
            if (oscillatorRef.current) {
                oscillatorRef.current.stop();
                oscillatorRef.current.disconnect();
            }
            if (audioContextRef.current && audioContextRef.current.state !== 'closed') {
                audioContextRef.current.close();
            }
        };
    }, []);

    return (
        <div className={`otamatone-container ${isReadOnly ? 'read-only' : ''}`}>
            <div 
                className="otamatone" 
                onMouseDown={handleMouseDown}
                style={{ cursor: isReadOnly ? 'default' : 'pointer' }}
            >
                <div className="otamatone-face">
                    <div className="otamatone-mouth" style={{
                        height: `${10 + (frequency / 2000) * 40}px`,
                        width: `${10 + (frequency / 2000) * 40}px`
                    }}></div>
                    <div className="otamatone-eyes">
                        <div className="eye left-eye"></div>
                        <div className="eye right-eye"></div>
                    </div>
                </div>
                <div className="otamatone-neck" ref={sliderRef}>
                    <div className="frequency-indicator" style={{left: `${(frequency - 20) / 1980 * 100}%`}}></div>
                    <div className="note-marker" style={{left: '10%', pointerEvents: 'none'}}>♩</div>
                    <div className="note-marker" style={{left: '50%', pointerEvents: 'none'}}>♪</div>
                    <div className="note-marker" style={{left: '90%', pointerEvents: 'none'}}>♫</div>
                </div>
            </div>
        </div>
    )
}